# Transfer between string and object
# Author:walkureHHH
# Last modify:2020/10/14
import numpy

class latex_str_error(Exception):
    pass

def bmatrix2numpy(latex_str)->numpy.array:
    latex_str = latex_str.strip()
    if latex_str.startswith('\\begin{bmatrix}') and latex_str.endswith('\\end{bmatrix}'):
        latex_str = latex_str.replace('\\begin{bmatrix}','').replace('\\end{bmatrix}','')
        rows = latex_str.split('\\\\')
        lis = [i.split('&') for i in rows]
        return numpy.array(lis)
    else:
        raise latex_str_error("this is not a bmatrix,please check it again")

def numpy2bmatrix(array)->str:
    s = '\\begin{bmatrix}'
    shape = numpy.shape(array)
    str_list = array.tolist()
    for i in range(0,shape[0]):
        for j in range(0,shape[1]):
            if j < shape[1]-1:
                s += (str(str_list[i][j])+'&')
            else:
                s += str(str_list[i][j])
        if i < shape[0]-1:
            s += '\\\\'
        else:
            s += '\\end{bmatrix}'
    return s

def str2list(list_str,mod_str = ',')->list:
    list_str = list_str.strip().replace('\n',mod_str)
    return list_str.split(mod_str)

def list2str(list_,mod_str=',')->str:
    result = ''
    for i in range(0,len(list_)):
        if i < len(list_)-1:
            result += (str(list_[i])+mod_str)
        else:
            result += str(list_[i])
    return "[%s]"%result

if __name__ == "__main__":
    lista = [1,2,3,4,5,6]
    print(lista)
    print(list2str(lista))



# a multithreading m3u8 download module and the number of threads can decide by yourself
# author: walkureHHH
# last modify: 2020/06/17
import requests
from threading import Thread
from threading import Lock
import os
import shutil
from tqdm import tqdm
class thread_num_ERROR(Exception):
    pass
class mod_ERROR(Exception):
    pass
class m3u8_downloader:
    temp_file_path = ''
    mp4_path = ''
    num_of_threads = ''
    m3u8_file_path = ''
    urls = []
    names = []
    has_download_name = []
    cant_dow = []
    total = 0
    lock = Lock()
    def __init__(self,m3u8_file_path,temp_file_path='.',mp4_path='./test.mp4',num_of_threads=10):
        if num_of_threads <= 0:
            raise thread_num_ERROR('the number of threads can\'t smaller than 0')
        self.mp4_path = mp4_path
        self.temp_file_path = temp_file_path 
        self.num_of_threads = num_of_threads
        self.m3u8_file_path = m3u8_file_path
        if os.path.exists(self.temp_file_path+'/TS'):
            print("""warning: the temporary folder has exited\n 
please comfirm the temporary folder included the fragment video you need""")
            self.has_download_name = os.listdir(self.temp_file_path+'/TS')
        else:
            os.mkdir(self.temp_file_path+'/TS')
            self.has_download_name = []
        with open(self.m3u8_file_path,'r') as m3u8:
            temp_url = [m3u8_lines.replace('\n','') for m3u8_lines in m3u8.readlines() if m3u8_lines.startswith('http')]
        self.total = len(temp_url)
        self.names = [i.split('/')[-1].split('?')[0] for i in temp_url]
        self.urls = [[] for j in range(0, self.num_of_threads)]
        for index, el in enumerate(temp_url):
            self.urls[index%self.num_of_threads].append(el)
        return
    
    def start(self,mod = 0, time_out = 60):
        if mod not in [0,1,2,3]:
            raise mod_ERROR('Only have mod 0 , 1 , 2 or 3')
        with tqdm(total=self.total,bar_format='<<*>> {percentage:3.0f}% {n_fmt}/{total_fmt} [{elapsed}<{remaining}] <<*>> ') as jdt:
            Threads = []
            for i in range(self.num_of_threads):
                thread = Thread(target=self.__download, args=(self.urls[i],'thread'+str(i),jdt,time_out))
                Threads.append(thread)
            for threads in Threads:
                threads.start()
            for threads in Threads:
                threads.join()
        percent = '%.02f%%'%((len(self.has_download_name)/len(self.names))*100)
        if len(self.has_download_name)==len(self.names):
            print('downloading finished',percent)
            for names in self.names:
                ts = open(self.temp_file_path+'/TS/'+names,'rb')
                with open(self.mp4_path,'ab') as mp4:
                    mp4.write(ts.read())
                ts.close()
            if mod == 0 or mod == 1:
                os.remove(self.m3u8_file_path)
            if mod == 0 or mod == 2:
                shutil.rmtree(self.temp_file_path+'/TS')
        else:
            print('----------------------------------------------------------------')
            for cantdow_urls in self.cant_dow:
                print('downloading fail:',cantdow_urls)
            print('incomplete downloading',percent)

    def __download(self, download_list, thread_name, jdt, time_out):
        for urls in download_list:
            if urls.split('/')[-1].split('?')[0] not in self.has_download_name:
                for i in range(0,5):
                    try:
                        conn = requests.get(urls,timeout=time_out)
                        if conn.status_code == 200:
                            with open(self.temp_file_path+'/TS/'+urls.split('/')[-1].split('?')[0],'wb') as ts:
                                ts.write(conn.content)
                            with self.lock:
                                if i != 0:
                                    print('\n'+thread_name,'redownload successfully',urls.split('/')[-1].split('?')[0])
                                self.has_download_name.append(urls.split('/')[-1].split('?')[0])
                                jdt.update(1)
                            break
                        else:
                            with self.lock:
                                if i == 0:
                                    print('\n'+thread_name,conn.status_code,urls.split('/')[-1].split('?')[0],'begin retry 1')
                                else:
                                    print('\n'+thread_name,conn.status_code,urls.split('/')[-1].split('?')[0],'Retry '+ str(i) +'/3')
                                if i == 4:
                                    self.cant_dow.append(urls)
                    except:
                        with self.lock:
                            if i == 0:
                                print('\n'+thread_name,'Time out ERROR',urls.split('/')[-1].split('?')[0],'begin retry 1')
                            else:
                                print('\n'+thread_name,'Time out ERROR',urls.split('/')[-1].split('?')[0],'Retry '+ str(i) +'/3')
                            if i == 4:
                                self.cant_dow.append(urls)
            else:
                with self.lock:
                    jdt.update(1)

if __name__ == "__main__":
    a = m3u8_downloader('/mnt/c/Users/kylis/Downloads/r.m3u8',temp_file_path='.',mp4_path='./1.mp4', num_of_threads=17)
    a.start()



def sort_dic_by_value(dic,my_reverse=False):
    return {k: v for k, v in sorted (dic.items(), key=lambda item: item[1], reverse=my_reverse)}

def sort_multi_list(*multi_list,by = 1,my_reverse=False):
    li = zip(*multi_list)
    length = len(multi_list)
    if length == 1:
        raise Exception("please input multiple lists, not just 1")
    else:
        if by in range(0,length):
            x = sorted(li, key=lambda i:i[by],reverse=my_reverse)
            return x
        else:
            raise Exception("the value of 'by' can only be 0 or 1")

def merge_list_to_dic(list1,list2):
    li = zip(list1,list2)
    return {k:v for (k,v) in li}

if __name__=="__main__":
    a = ["a","b","c","d"]
    b = [1,2,3,4]
    c = [3,4,7,5]
    dic = merge_list_to_dic(a,b)
    print(sort_dic_by_value(dic,True))
    print(sort_multi_list(a,b,c,by=2,my_reverse=True))


import time
import pickle
import os

# def mkprj(name="flavour_wheel_test", structure=
# ["logs","results/pic_results","results/linkage_results","results/web_results","results/word_results","models/glove"]):
#     # os.mkdir(name)
#     for i in structure:
#         os.makedirs(os.path.join(name ,i))

def time_string():
    return time.strftime("%Y-%m-%d-%H%M%S", time.localtime())

def load_result(path_name):
    with open(path_name, "rb") as f:
        return pickle.load(f)
    
def store_result(path_name, Aobject):
    path = os.path.dirname(path_name)
    if os.path.exists(path)==False:
        os.makedirs(path)
    with open(path_name, "wb") as f:
        pickle.dump(Aobject, f)

def print_list(Alist, num_of_columns, separator_in_line=" , ", separator_between_line="\n"):
    length = len(Alist)
    for i in range(length):
        if i%num_of_columns != (num_of_columns-1):
            if i == (length-1):
                print(Alist[i], end = separator_between_line)
            else:
                print(Alist[i], end = separator_in_line)
        else:
            print(Alist[i], end = separator_between_line)
    print("\nlength: %d"%length)